@@ -29,7 +29,7 @@
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE. */
 
-/* $Id: wdt.h 2522 2016-04-20 05:43:23Z joerg_wunsch $ */
+/* $Id$ */
 
 /*
    avr/wdt.h - macros for AVR watchdog timer
@@ -72,10 +72,10 @@
 
     uint8_t mcusr_mirror __attribute__ ((section (".noinit")));
 
-    void get_mcusr(void) \
-      __attribute__((naked)) \
-      __attribute__((section(".init3")));
-    void get_mcusr(void)
+    __attribute__((used, unused, naked, section(".init3")))
+    static void get_mcusr (void);
+
+    void get_mcusr (void)
     {
       mcusr_mirror = MCUSR;
       MCUSR = 0;
@@ -100,6 +100,10 @@
 
 #ifndef __DOXYGEN__
 
+#ifndef __ATTR_ALWAYS_INLINE__
+#define __ATTR_ALWAYS_INLINE__ __inline__ __attribute__((__always_inline__))
+#endif
+
 #if defined(WDP3)
 # define _WD_PS3_MASK       _BV(WDP3)
 #else
@@ -122,7 +126,7 @@
 
 #endif	/* !__DOXYGEN__ */
 
-
+#ifdef __DOXYGEN__
 /**
    \ingroup avr_watchdog
    Enable the watchdog timer, configuring it for expiry after
@@ -133,10 +137,55 @@
 
    See also the symbolic constants \c WDTO_15MS et al.
 */
+#define wdt_enable(timeout)
+#endif	/* __DOXYGEN__ */
 
 
 #if defined(__AVR_XMEGA__)
 
+#if defined (WDT_CTRLA) && !defined(RAMPD)
+
+#define wdt_enable(timeout) \
+do { \
+uint8_t __temp; \
+__asm__ __volatile__ ( \
+	"wdr"									"\n\t" \
+	"out %[ccp_reg], %[ioreg_cen_mask]"		"\n\t" \
+	"lds %[tmp], %[wdt_reg]"				"\n\t" \
+	"sbr %[tmp], %[wdt_enable_timeout]"		"\n\t" \
+	"sts %[wdt_reg], %[tmp]"				"\n\t" \
+	"1:lds %[tmp], %[wdt_status_reg]"		"\n\t" \
+	"sbrc %[tmp], %[wdt_syncbusy_bit]"		"\n\t" \
+	"rjmp 1b"								       \
+	: [tmp]					"=d" (__temp) \
+	: [ccp_reg]				"I"  (_SFR_IO_ADDR(CCP)), \
+	  [ioreg_cen_mask]		"r"  ((uint8_t)CCP_IOREG_gc), \
+	  [wdt_reg]				"n"  (_SFR_MEM_ADDR(WDT_CTRLA)), \
+	  [wdt_enable_timeout]	"M"  (timeout), \
+	  [wdt_status_reg]		"n"  (_SFR_MEM_ADDR(WDT_STATUS)), \
+	  [wdt_syncbusy_bit]	"I"  (WDT_SYNCBUSY_bm) \
+); \
+} while(0)
+
+#define wdt_disable() \
+do { \
+uint8_t __temp; \
+__asm__ __volatile__ (  \
+	"wdr"								"\n\t" \
+	"out %[ccp_reg], %[ioreg_cen_mask]" "\n\t" \
+	"lds %[tmp], %[wdt_reg]"			"\n\t" \
+	"cbr %[tmp], %[timeout_mask]"       "\n\t" \
+	"sts %[wdt_reg], %[tmp]"			       \
+    : [tmp]            "=d" (__temp) \
+    : [ccp_reg]        "I" (_SFR_IO_ADDR(CCP)),       \
+      [ioreg_cen_mask] "r" ((uint8_t)CCP_IOREG_gc),   \
+      [wdt_reg]        "n" (_SFR_MEM_ADDR(WDT_CTRLA)),\
+      [timeout_mask]   "I" (WDT_PERIOD_gm) \
+); \
+} while(0)
+
+#else // defined (WDT_CTRLA) && !defined(RAMPD)
+
 /*
    wdt_enable(timeout) for xmega devices
 ** write signature (CCP_IOREG_gc) that enables change of protected I/O
@@ -151,7 +200,7 @@
 */
 #define wdt_enable(timeout) \
 do { \
-uint8_t temp; \
+uint8_t __temp; \
 __asm__ __volatile__ (         \
     "in __tmp_reg__, %[rampd]"              "\n\t" \
     "out %[rampd], __zero_reg__"            "\n\t" \
@@ -160,8 +209,8 @@ __asm__ __volatile__ (         \
     "1:lds %[tmp], %[wdt_status_reg]"       "\n\t" \
     "sbrc %[tmp], %[wdt_syncbusy_bit]"      "\n\t" \
     "rjmp 1b"                               "\n\t" \
-    "out %[rampd], __tmp_reg__"             "\n\t" \
-    : [tmp]                "=r" (temp) \
+    "out %[rampd], __tmp_reg__"                    \
+    : [tmp]                "=r" (__temp) \
     : [rampd]              "I" (_SFR_IO_ADDR(RAMPD)),      \
       [ccp_reg]            "I" (_SFR_IO_ADDR(CCP)),        \
       [ioreg_cen_mask]     "r" ((uint8_t)CCP_IOREG_gc),     \
@@ -179,15 +228,17 @@ __asm__ __volatile__ (  \
     "out %[rampd], __zero_reg__"        "\n\t" \
     "out %[ccp_reg], %[ioreg_cen_mask]" "\n\t" \
     "sts %[wdt_reg], %[disable_mask]"   "\n\t" \
-    "out %[rampd], __tmp_reg__"         "\n\t" \
-    : \
+    "out %[rampd], __tmp_reg__"                \
+    : /* no outputs */ \
     : [rampd]             "I" (_SFR_IO_ADDR(RAMPD)),    \
       [ccp_reg]           "I" (_SFR_IO_ADDR(CCP)),      \
       [ioreg_cen_mask]    "r" ((uint8_t)CCP_IOREG_gc),   \
       [wdt_reg]           "n" (_SFR_MEM_ADDR(WDT_CTRL)), \
       [disable_mask]      "r" ((uint8_t)((~WDT_ENABLE_bm) | WDT_CEN_bm)) \
     : "r0" \
-);
+)
+
+#endif // defined (WDT_CTRLA) && !defined(RAMPD)
 
 #elif defined(__AVR_TINY__)
 
@@ -198,7 +249,7 @@ __asm__ __volatile__ ( \
     "wdr" "\n\t"  \
     "out %[CCPADDRESS],%[SIGNATURE]" "\n\t"  \
     "out %[WDTREG],%[WDVALUE]" "\n\t"  \
-    "out __SREG__,__tmp_reg__" "\n\t"  \
+    "out __SREG__,__tmp_reg__"         \
     : /* no outputs */  \
     : [CCPADDRESS] "I" (_SFR_IO_ADDR(CCP)),  \
       [SIGNATURE] "r" ((uint8_t)0xD8), \
@@ -210,7 +261,7 @@ __asm__ __volatile__ ( \
 
 #define wdt_disable() \
 do { \
-uint8_t temp_wd; \
+uint8_t __temp_wd; \
 __asm__ __volatile__ ( \
     "in __tmp_reg__,__SREG__" "\n\t"  \
     "cli" "\n\t"  \
@@ -219,21 +270,19 @@ __asm__ __volatile__ ( \
     "in  %[TEMP_WD],%[WDTREG]" "\n\t" \
     "cbr %[TEMP_WD],%[WDVALUE]" "\n\t" \
     "out %[WDTREG],%[TEMP_WD]" "\n\t" \
-    "out __SREG__,__tmp_reg__" "\n\t" \
-    : /*no output */ \
+    "out __SREG__,__tmp_reg__"        \
+    : [TEMP_WD] "=d" (__temp_wd) \
     : [CCPADDRESS] "I" (_SFR_IO_ADDR(CCP)), \
       [SIGNATURE] "r" ((uint8_t)0xD8), \
       [WDTREG] "I" (_SFR_IO_ADDR(_WD_CONTROL_REG)), \
-      [TEMP_WD] "d" (temp_wd), \
       [WDVALUE] "n" (1 << WDE) \
     : "r16" \
 ); \
-}while(0)
+} while(0)
 
 #elif defined(CCP)
 
-static __inline__
-__attribute__ ((__always_inline__))
+static __ATTR_ALWAYS_INLINE__
 void wdt_enable (const uint8_t value)
 {
 	if (!_SFR_IO_REG_P (CCP) && !_SFR_IO_REG_P (_WD_CONTROL_REG))
@@ -244,7 +293,7 @@ void wdt_enable (const uint8_t value)
 			"wdr" "\n\t"
 			"sts %[CCPADDRESS],%[SIGNATURE]" "\n\t"
 			"sts %[WDTREG],%[WDVALUE]" "\n\t"
-			"out __SREG__,__tmp_reg__" "\n\t"
+			"out __SREG__,__tmp_reg__"
 			: /* no outputs */
 			: [CCPADDRESS] "n" (_SFR_MEM_ADDR(CCP)),
 			[SIGNATURE] "r" ((uint8_t)0xD8),
@@ -262,7 +311,7 @@ void wdt_enable (const uint8_t value)
 			"wdr" "\n\t"
 			"sts %[CCPADDRESS],%[SIGNATURE]" "\n\t"
 			"out %[WDTREG],%[WDVALUE]" "\n\t"
-			"out __SREG__,__tmp_reg__" "\n\t"
+			"out __SREG__,__tmp_reg__"
 			: /* no outputs */
 			: [CCPADDRESS] "n" (_SFR_MEM_ADDR(CCP)),
 			[SIGNATURE] "r" ((uint8_t)0xD8),
@@ -280,7 +329,7 @@ void wdt_enable (const uint8_t value)
 			"wdr" "\n\t"
 			"out %[CCPADDRESS],%[SIGNATURE]" "\n\t"
 			"sts %[WDTREG],%[WDVALUE]" "\n\t"
-			"out __SREG__,__tmp_reg__" "\n\t"
+			"out __SREG__,__tmp_reg__"
 			: /* no outputs */
 			: [CCPADDRESS] "I" (_SFR_IO_ADDR(CCP)),
 			[SIGNATURE] "r" ((uint8_t)0xD8),
@@ -298,7 +347,7 @@ void wdt_enable (const uint8_t value)
 			"wdr" "\n\t"
 			"out %[CCPADDRESS],%[SIGNATURE]" "\n\t"
 			"out %[WDTREG],%[WDVALUE]" "\n\t"
-			"out __SREG__,__tmp_reg__" "\n\t"
+			"out __SREG__,__tmp_reg__"
 			: /* no outputs */
 			: [CCPADDRESS] "I" (_SFR_IO_ADDR(CCP)),
 			[SIGNATURE] "r" ((uint8_t)0xD8),
@@ -310,13 +359,12 @@ void wdt_enable (const uint8_t value)
 	}
 }
 
-static __inline__
-__attribute__ ((__always_inline__))
+static __ATTR_ALWAYS_INLINE__
 void wdt_disable (void)
 {
 	if (!_SFR_IO_REG_P (CCP) && !_SFR_IO_REG_P(_WD_CONTROL_REG))
 	{
-        uint8_t temp_wd;
+        uint8_t __temp_wd;
         __asm__ __volatile__ (
 				"in __tmp_reg__,__SREG__" "\n\t"
 				"cli" "\n\t"
@@ -325,19 +373,18 @@ void wdt_disable (void)
 				"lds %[TEMP_WD],%[WDTREG]" "\n\t"
 				"cbr %[TEMP_WD],%[WDVALUE]" "\n\t"
 				"sts %[WDTREG],%[TEMP_WD]" "\n\t"
-				"out __SREG__,__tmp_reg__" "\n\t"
-				: /*no output */
+				"out __SREG__,__tmp_reg__"
+				: [TEMP_WD] "=d" (__temp_wd)
 				: [CCPADDRESS] "n" (_SFR_MEM_ADDR(CCP)),
 				[SIGNATURE] "r" ((uint8_t)0xD8),
 				[WDTREG] "n" (_SFR_MEM_ADDR(_WD_CONTROL_REG)),
-				[TEMP_WD] "d" (temp_wd),
 				[WDVALUE] "n" (1 << WDE)
 				: "r0"
 				);
 	}
 	else if (!_SFR_IO_REG_P (CCP) && _SFR_IO_REG_P(_WD_CONTROL_REG))
 	{
-        uint8_t temp_wd;
+        uint8_t __temp_wd;
         __asm__ __volatile__ (
 				"in __tmp_reg__,__SREG__" "\n\t"
 				"cli" "\n\t"
@@ -346,19 +393,18 @@ void wdt_disable (void)
 				"in %[TEMP_WD],%[WDTREG]" "\n\t"
 				"cbr %[TEMP_WD],%[WDVALUE]" "\n\t"
 				"out %[WDTREG],%[TEMP_WD]" "\n\t"
-				"out __SREG__,__tmp_reg__" "\n\t"
-				: /*no output */
+				"out __SREG__,__tmp_reg__"
+				: [TEMP_WD] "=d" (__temp_wd)
 				: [CCPADDRESS] "n" (_SFR_MEM_ADDR(CCP)),
 				[SIGNATURE] "r" ((uint8_t)0xD8),
 				[WDTREG] "I" (_SFR_IO_ADDR(_WD_CONTROL_REG)),
-				[TEMP_WD] "d" (temp_wd),
 				[WDVALUE] "n" (1 << WDE)
 				: "r0"
 				);
 	}
 	else if (_SFR_IO_REG_P (CCP) && !_SFR_IO_REG_P(_WD_CONTROL_REG))
 	{
-        uint8_t temp_wd;
+        uint8_t __temp_wd;
         __asm__ __volatile__ (
 				"in __tmp_reg__,__SREG__" "\n\t"
 				"cli" "\n\t"
@@ -367,19 +413,18 @@ void wdt_disable (void)
 				"lds %[TEMP_WD],%[WDTREG]" "\n\t"
 				"cbr %[TEMP_WD],%[WDVALUE]" "\n\t"
 				"sts %[WDTREG],%[TEMP_WD]" "\n\t"
-				"out __SREG__,__tmp_reg__" "\n\t"
-				: /*no output */
+				"out __SREG__,__tmp_reg__"
+				: [TEMP_WD] "=d" (__temp_wd)
 				: [CCPADDRESS] "I" (_SFR_IO_ADDR(CCP)),
 				[SIGNATURE] "r" ((uint8_t)0xD8),
 				[WDTREG] "n" (_SFR_MEM_ADDR(_WD_CONTROL_REG)),
-				[TEMP_WD] "d" (temp_wd),
 				[WDVALUE] "n" (1 << WDE)
 				: "r0"
 				);
 	}
 	else
 	{
-        uint8_t temp_wd;
+        uint8_t __temp_wd;
         __asm__ __volatile__ (
 				"in __tmp_reg__,__SREG__" "\n\t"
 				"cli" "\n\t"
@@ -388,12 +433,11 @@ void wdt_disable (void)
 				"in %[TEMP_WD],%[WDTREG]" "\n\t"
 				"cbr %[TEMP_WD],%[WDVALUE]" "\n\t"
 				"out %[WDTREG],%[TEMP_WD]" "\n\t"
-				"out __SREG__,__tmp_reg__" "\n\t"
-				: /*no output */
+				"out __SREG__,__tmp_reg__"
+				: [TEMP_WD] "=d" (__temp_wd)
 				: [CCPADDRESS] "I" (_SFR_IO_ADDR(CCP)),
 				[SIGNATURE] "r" ((uint8_t)0xD8),
 				[WDTREG] "I" (_SFR_IO_ADDR(_WD_CONTROL_REG)),
-				[TEMP_WD] "d" (temp_wd),
 				[WDVALUE] "n" (1 << WDE)
 				: "r0"
 				);
@@ -402,8 +446,7 @@ void wdt_disable (void)
 
 #else
 
-static __inline__
-__attribute__ ((__always_inline__))
+static __ATTR_ALWAYS_INLINE__
 void wdt_enable (const uint8_t value)
 {
 	if (_SFR_IO_REG_P (_WD_CONTROL_REG))
@@ -414,7 +457,7 @@ void wdt_enable (const uint8_t value)
 				"wdr" "\n\t"
 				"out %0, %1" "\n\t"
 				"out __SREG__,__tmp_reg__" "\n\t"
-				"out %0, %2" "\n \t"
+				"out %0, %2"
 				: /* no outputs */
 				: "I" (_SFR_IO_ADDR(_WD_CONTROL_REG)),
 				"r" ((uint8_t)(_BV(_WD_CHANGE_BIT) | _BV(WDE))),
@@ -431,7 +474,7 @@ void wdt_enable (const uint8_t value)
 				"wdr" "\n\t"
 				"sts %0, %1" "\n\t"
 				"out __SREG__,__tmp_reg__" "\n\t"
-				"sts %0, %2" "\n \t"
+				"sts %0, %2"
 				: /* no outputs */
 				: "n" (_SFR_MEM_ADDR(_WD_CONTROL_REG)),
 				"r" ((uint8_t)(_BV(_WD_CHANGE_BIT) | _BV(WDE))),
@@ -442,13 +485,12 @@ void wdt_enable (const uint8_t value)
 	}
 }
 
-static __inline__
-__attribute__ ((__always_inline__))
+static __ATTR_ALWAYS_INLINE__
 void wdt_disable (void)
 {
 	if (_SFR_IO_REG_P (_WD_CONTROL_REG))
 	{
-        uint8_t register temp_reg;
+        uint8_t __temp_reg;
 		__asm__ __volatile__ (
 				"in __tmp_reg__,__SREG__"    "\n\t"
 				"cli"                        "\n\t"
@@ -457,8 +499,8 @@ void wdt_disable (void)
 				"ori %[TEMPREG],%[WDCE_WDE]" "\n\t"
 				"out %[WDTREG],%[TEMPREG]"   "\n\t"
 				"out %[WDTREG],__zero_reg__" "\n\t"
-				"out __SREG__,__tmp_reg__"   "\n\t"
-				: [TEMPREG] "=d" (temp_reg)
+				"out __SREG__,__tmp_reg__"
+				: [TEMPREG] "=d" (__temp_reg)
 				: [WDTREG]  "I"  (_SFR_IO_ADDR(_WD_CONTROL_REG)),
 				[WDCE_WDE]  "n"  ((uint8_t)(_BV(_WD_CHANGE_BIT) | _BV(WDE)))
 				: "r0"
@@ -466,7 +508,7 @@ void wdt_disable (void)
 	}
 	else
 	{
-        uint8_t register temp_reg;
+        uint8_t __temp_reg;
 		__asm__ __volatile__ (
 				"in __tmp_reg__,__SREG__"    "\n\t"
 				"cli"                        "\n\t"
@@ -475,8 +517,8 @@ void wdt_disable (void)
 				"ori %[TEMPREG],%[WDCE_WDE]" "\n\t"
 				"sts %[WDTREG],%[TEMPREG]"   "\n\t"
 				"sts %[WDTREG],__zero_reg__" "\n\t"
-				"out __SREG__,__tmp_reg__"   "\n\t"
-				: [TEMPREG] "=d" (temp_reg)
+				"out __SREG__,__tmp_reg__"
+				: [TEMPREG] "=d" (__temp_reg)
 				: [WDTREG]  "n"  (_SFR_MEM_ADDR(_WD_CONTROL_REG)),
 				[WDCE_WDE]  "n"  ((uint8_t)(_BV(_WD_CHANGE_BIT) | _BV(WDE)))
 				: "r0"
@@ -547,9 +589,8 @@ void wdt_disable (void)
     ATtiny24, ATtiny44, ATtiny84, ATtiny84A,
     ATtiny25, ATtiny45, ATtiny85, 
     ATtiny261, ATtiny461, ATtiny861, 
-    ATmega48, ATmega88, ATmega168,
-    ATmega48P, ATmega88P, ATmega168P, ATmega328P,
-    ATmega164P, ATmega324P, ATmega644P, ATmega644,
+    ATmega48*, ATmega88*, ATmega168*, ATmega328*,
+    ATmega164P, ATmega324P, ATmega324PB, ATmega644P, ATmega644,
     ATmega640, ATmega1280, ATmega1281, ATmega2560, ATmega2561,
     ATmega8HVA, ATmega16HVA, ATmega32HVB,
     ATmega406, ATmega1284P,
@@ -572,9 +613,8 @@ void wdt_disable (void)
     ATtiny24, ATtiny44, ATtiny84, ATtiny84A,
     ATtiny25, ATtiny45, ATtiny85, 
     ATtiny261, ATtiny461, ATtiny861, 
-    ATmega48, ATmega48A, ATmega48PA, ATmega88, ATmega168,
-    ATmega48P, ATmega88P, ATmega168P, ATmega328P,
-    ATmega164P, ATmega324P, ATmega644P, ATmega644,
+    ATmega48*, ATmega88*, ATmega168*, ATmega328*,
+    ATmega164P, ATmega324P, ATmega324PB, ATmega644P, ATmega644,
     ATmega640, ATmega1280, ATmega1281, ATmega2560, ATmega2561,
     ATmega8HVA, ATmega16HVA, ATmega32HVB,
     ATmega406, ATmega1284P,
